import sys

from TaskEngine import Utils
from TaskEngine.Engine import IEngine, Engine, Log, Task, Pools, Loader, TaskRunner
        
class ZopeTaskPool(Pools.GraphTaskPool):
    pass

class IZopeTask(Task.ITask):

    # 4 compatiblity, newer versions use mapply, instead
    # of getting args out of contexts.
    
    def __call__(context, persistent_object):
        """
        perform the action related to this task,
        args are the configuration object and
        a persistent object. the persistent object
        is either the result of a task we are dependenant
        on or the zodb root. if we return a persistent
        object, than it will be the persistent object
        arg to tasks which ask for it by our task id.
        """
        
class ZopeFunctionTask(Task.FunctionTask):

    __implements__ = IZopeTask

    type = 'Zope'

    def __call__(self, context, persistent_object):
        return apply(self.func, (context, persistent_object))

Task.registerTask(ZopeFunctionTask, "OldEngineCompatibleZopeTask")

def taskSelector(load_context, task_context, object):
    
    # the old style explictly referenced type within the id
    context_id = task_context.getId()
    task_type, task_id  = context_id.split('Task-')
    task_context['id']=task_id
    
    return getTask('OldEngineCompatibleZopeTask')    

Task.taskSelector = taskSelector

class ZopeTaskRunner(TaskRunner):
    """

    this is a task runner that executes tasks with the signature of the old
    zope tasks, ie func(config, persistent_object), where persistent_object
    can be specified by objectbytaskid in the task config, or the root
    object if none is specified.
    """

    def __init__(self, config, root):
        self.config = config
        self._return_registry = {}
        self.root = root

    def executeTasks(self, execution_context, tasks):

        from AccessControl import getSecurityManager
        log = getLog(self)

        log.info('Beginning Execution %d Tasks', len(tasks))
        for t in tasks:

            log.info('Executing %s', t)
            
            tconf = t.getContext()
            otid = tconf.get('objectbytaskid', None)

            if otid is not None:
                persistent_object = self._return_registry.get(otid, None)
                if persistent_object is None:
                    raise TaskRunnerException (
                        " ZopeTask %s Dependant on NonExistent Persistent Object "%str(t)
                        )
            else:
                persistent_object = self.root
            try:
                r = t(tconf, persistent_object)
            except:
                log.critical('FAILED %s', t.getId())
                r = None
                raise
            self._return_registry[t.getId()]=r

        # commit the transaction
        u = getSecurityManager().getUser()
        t = get_transaction()
        t.setUser(u)
        t.note('site installation')
        t.commit()
        
        log.info('Finished Executing %d Tasks', len(tasks))    

Pools.registerPool( ZopeTaskPool, 'OldEngineCompatibleZopePool')

class ZopeEngine(Engine):

    __implements__ = IEngine

    def configure(self, **kwargs):
        
        Engine.configure(self, **kwargs)
        engine_ctx_chain = self.getContext()
        defaults_ctx = engine_ctx_chain.getContext('defaults')
        
        defaults_ctx['DefaultPoolType']='OldZopePool'
        defaults_ctx['TaskFilePattern']='*.txt'

        
    def loadTasks(self):
        """
        load the specified tasks

        there is alot of work to be done in cleaning this up,
        i took the shortcut, of just putting hacks in here to get
        things working, till i have time to clean up. at least
        most of the hacks are localized to this function...
        """

        context = self.getContext()
        log = Log.getLog(self)
        log.info('loading tasks from %s'%str(context.SourceDirectory))

        # XXX we do a hack to allow for zope instance homes
        # this should get pushed down to the loader or pool,
        # ideally this would be on the pool, but the default
        # loader loads tasks before adding/creating the pool

        log.debug('loading zope environment')
        Utils.zope_setup_load_env(context)
        try:
            pools = Loader.MultiPoolLoader().load(context)
        finally:
            possible_bad = Utils.zope_clean_load_env(context)

        pools = Utils.sort_by_attr(pools, 'priority')

        runner_factory = TaskRunner.ZopeTaskRunner
        log.debug('importing zope')

        import Zope
        r = Zope.app() 

        #################################
        # This is painfully ugly, and rather baffling.
        # these trix weren't needed in the original installer
        # ICK!
        #################################

        import Products
        located = dir(Products)
        bad = [pb for pb in possible_bad if pb not in located]
        for b in bad:
            del sys.modules['Products.%s'%b]

        for b in bad:
            __import__('Products.%s'%b, globals(), locals(), [])                

        #################################
        for p in pools:
            p.setTaskRunner(runner_factory({},r))
            
        pools  = map(Task.PoolTask, pools)
        
        # XXX again, since we don't have a config step at the moment
        # for pool contexts to be setup, we explicitly set it
        # it to the entire load context, we are creating lots
        # of leaks most likely :-(...
        map(lambda p, c=context: p.setContext(c), pools)
        self.tasks = Pools.SetTaskPool('TaskPoolSet')
        map(self.tasks.addTask, pools)
        log.info('%d task pools loaded', len(self.tasks))
          
run = ZopeEngine().ignition
