"""
  Purpose: CMF Tasks
  Author: kapil <kapil@sanhawk.net>
  Created: 7/11/2002
  CVS: $Id: CMFTasks.py,v 1.38 2002/09/11 07:44:04 kapil Exp $
"""

def install_tool(config, cmf):
    
    productmodule = config.productmodule.strip()
    from_   = config['toolclass'].strip()

    obj = __import__(productmodule, globals(), locals(), [from_])
    obj = getattr(obj, from_)    
    obj = obj()

    cmf.portal_url.getPortalObject()

    cmf._setObject(obj.getId(), obj)

def add_properties(config, object):
    "add properties to object"
    props = [p for p in config.keys() if p.startswith('property')]
    for p in props:
        p = config.get(p)
        p = filter(None, [x.strip() for x in p.split(',')])
        type, field, value = p
        # config tool escapes \n, turn into newline
        value = map(lambda x: x.strip(), value.split('\\n'))
        value = '\n'.join(value)        
        object.manage_addProperty(field, value, type)

def install_skin(config, cmf):
    skinproduct = config.skinproduct.strip()
    skincomponents = map(lambda x: x.strip(), config.skincomponents.split(','))
    skin_tool = cmf.portal_skins
    default_sid = skins.getDefaultSkin()
    skin_map = skin_tool._getSelections()
    default_skin = skin_map[default_sid]
    path_components = [p.strip() for p in default_skin.split(',')]
    
    for sc in skincomponents:
        skin_tool.manage_addProduct['CMFCore'].manage_addDirectoryView(
            '/'.join( (skinproduct, sc) )
            )
    if path_components.count('custom') > 0:
        path_components.remove('custom')

    merged_skin = skincomponents.extend(path_components)
    merged_skin.insert(0, 'custom')
    skin_map[default_sid]=', '.join(merged_skin)

def set_content_workflow(config, cmf):

    ctype  = [config.contenttype.strip()]
    chain  = config.workflowchain.strip()
 
    wf_tool = cmf.portal_workflow 
    wf_tool.setChainForPortalTypes(ctype, chain)

def install_directory_view(config, folder):

    directory_view_path = config.directory_view_path
    
    folder.manage_addProduct['CMFCore'].manage_addDirectoryView(
        filepath = directory_view_path
        )

def install_skin_directory_view(config, folder):

    skins = folder.portal_url.getPortalObject().portal_skins
    install_directory_view(config, skins)
        
def install_view_on_default_skin(config, folder):

    view_name = config.view_name.strip()
    skins = folder.portal_url.getPortalObject().portal_skins
    
    skin_name = skins.getDefaultSkin()
    skin_path = skins.getSkinPath(skin_name)
    components = map(lambda x: x.strip(), skin_path.split(','))

    if not view_name in components:
        components.insert(1, view_name)
        new_skin_path = ', '.join(components)
        skins.addSkinSelection(skin_name, new_skin_path, make_default=1)

def install_cmf_folder(config, folder):

    folder.manage_addPortalFolder(config.container_id,
                               config.container_title)
    
    container = folder._getOb(config.container_id)
    return container

def install_cmf_root_folder(config, cmf):


    folder = install_cmf_folder(config, cmf)
    
    id = folder.getId()
    name = folder.title_or_id()

    if config.has_key('alternate_action'):
        #print 'has alternate action'
        action = "string: ${portal_url}/%s/%s"%(id,config.alternate_action)
    else:
        #print 'does not have alternate action'
        action = "string: ${portal_url}/%s"%id
        
    condition = ''
    permission = 'View'
    category = 'global_tabs'
    
    actions = folder.portal_actions

    actions.addAction(
        id = id,
        name = name,
        action = action,
        condition = condition,
        permission = permission,
        category = category
        )
    
    return folder

def remove_cmf_actions(config, cmf):
        actions_tool = cmf.portal_actions
        return remove_generic_actions(config,cmf, actions_tool)
        
def remove_membership_actions(config, cmf):
        actions_tool = cmf.portal_membership
        return remove_generic_actions(config,cmf, actions_tool)
    
def remove_generic_actions(config, cmf, actions_tool):

    aids = [aid.strip() for aid in config.actionids.split(',')]

    actions = actions_tool.listActions()
    
    action_indexes = []
    for i in range(len(actions)):
        a = actions[i]
        if a.Title() in aids:

            action_indexes.append(i)

    actions_tool.deleteActions(action_indexes)

def register_type_info(config, cmf):

    from Products.CMFCore.TypesTool import FactoryTypeInformation
    from Products.CMFCore.TypesTool import ScriptableTypeInformation

    if config.has_key('fti_type') and config['fti_type']=='Script':
        fti_type = ScriptableTypeInformation.meta_type
    else:
        fti_type = FactoryTypeInformation.meta_type
        
    type_id = config.type_id
    type_info_name = config.type_info_name
    
    types = cmf.portal_types
        
    types.manage_addTypeInformation(
        fti_type,
        id = type_id,
        typeinfo_name = type_info_name
        )

def register_metadata_element(config, cmf):

    def value(name, config=config):
        if config.has_key(name):
            return config[name]
        return None

    # the following two are required
    element = config.element.strip()
    is_multi_valued = int(config.is_multi_valued.strip())

    is_required = value('is_required') or 0
    is_required = int(is_required)
    supply_default = value('supply_default') or 0
    supply_default = int(supply_default)
    default_value  = value('default_value') or '' # hmmm
    default_value  = default_value.strip()
    enforce_vocabulary = value('enforce_vocabulary') or 0
    enforce_vocabulary = int(enforce_vocabulary)
    allowed_vocabulary = value('allowed_vocabulary') or ''
    allowed_vocabulary = map(lambda x: x.strip(), filter(None, allowed_vocabulary.split(',')))
    content_type = value('content_type') # None or <default> for default policy
    
    pm = cmf.portal_metadata

    pm.addElementSpec(
        element = element,
        is_multi_valued = is_multi_valued
        )

    element = pm.getElementSpec(element)
    try:
        policy = element.getPolicy(content_type)
    except:
        policy = None

    if policy is  None:
        policy.addPolicy(content_type)
        policy = element.getPolicy(content_type)
        
    policy.edit(
        is_required = is_required
        , supply_default = supply_default
        , default_value = default_value 
        , enforce_vocabulary = enforce_vocabulary
        , allowed_vocabulary = allowed_vocabulary
        )

def edit_topic(config, topic):
    "Add criteria to topic."
    # sigh, this part sucks, not general enough
    criterion = [c for c in config.keys() if c.startswith('criterion')]

    for c in criterion:
        c = config.get(c)
        c = filter(None, [x.strip() for x in c.split(',', 2)])
        criterion_type, field, value = c

        topic.addCriterion(field, criterion_type)
        criterion = topic.getCriterion('crit__%s'%field)

        # "List Criterion", field, operator|None, value (, value)*
        if criterion_type == 'List Criterion':
            operator, values = value.split(',', 1)
            if not operator or operator == 'None':
                operator = None
            values = map(lambda x: x.strip(), values.split(','))
            values = '\n'.join(values)
            criterion.edit(value=values, operator=operator)
        else:
            criterion.edit(value=value)
    return topic

def install_topic(config, folder):
    # this method below is actual generic content creation...
    topic = install_content_container(config, folder)
    return edit_topic(config, topic)

def install_subtopic(config, topic):
    subtopic = topic.addSubtopic(config.id)
    subtopic.title = config.title
    edit_topic(config, subtopic)

def push_workflow(config, ob):
    """
    pushes an object through a number of workflow actions,
    mainly for chaining together transitions
    """
    actions = filter(None,[a.strip() for a in config['actions'].split(',')])
    wf_tool = ob.portal_workflow
  
    for a in actions:
        wf_tool.doActionFor(ob, a)

def push_workflow_for_object(config, cmf):

    path = filter(None, [p.strip() for p in config.path.split(',')])
    ob = cmf
    for p in path:
        ob = getattr(ob, p)

    push_workflow(config, ob)
    
        
def add_action(config,folder):
    # adds an action in portal_actions
    id = config.action_id.strip()
    name = config.action_name.strip()
    condition = config.action_condition.strip()
    permission = config.action_permission.strip()
    category = config.action_category.strip()
    action=config.action_action.strip()

    actions = folder.portal_actions

    actions.addAction(
        id = id,
        name = name,
        action = action,
        condition = condition,
        permission = permission,
        category = category
        )

def add_static_page_template(config, cmf):
    
    import os
    prefix = os.environ['INSTANCE_HOME']
    file = config.file.strip()
    file= os.path.join(prefix, file)
    
    page = ""
    fileobject = open(file,'r')

    line = fileobject.readline()
    while(line != ""):
        page = page + line
        line = fileobject.readline()

    # Traverse down to the place where the object will live
    path = filter(None, [p.strip() for p in config.path.split(',')])
    ob = cmf
    for p in path:
        ob = getattr(ob,p)

    # Create the object
    ob.manage_addProduct['PageTemplates'].manage_addPageTemplate(config.zpt_id)
    theZPT = ob[config.zpt_id]
    theZPT.pt_setTitle(config.zpt_title)
    theZPT.write(page)

    return theZPT

def add_external_method(config, parent):

    ob = parent

    # Create the object
    try:
      ob.manage_addProduct['ExternalMethod'].manage_addExternalMethod(id=config.em_id,title='',module=config.em_module,function=config.em_function,REQUEST=None)
    except:
      pass

    theEM = ob[config.em_id]

    return theEM

def add_catalog_index_metadata(config, cmf):
    "add an index and metadata field to the portal catalog"
    cmf.portal_catalog.manage_addColumn(config.id)
    method = getattr(cmf.portal_catalog.manage_addProduct['PluginIndexes'],
                     "manage_add" + config.index)
    method(id = config.id)


