import os
from os import path
from os.path import join, walk, isdir, splitext

def visitor(all_files, directory, directory_files):
    
    if directory.endswith('CVS'):
        return
    
    for f in directory_files:
        fqn = join(directory, f)
        if isdir(fqn):
            continue
        all_files.append(fqn)
    
def gather(directory):
    files = []
    walk(directory, visitor, files)
    return files
    
    
_extension_handlers = {}

def register_handler(ext, handler):
    global _extension_handlers
    _extension_handlers[ext]=handler

def make_hash_lookup(sequence):
    from operator import setitem
    d = {}
    map(lambda x, d=d, s=setitem: s(d,x,None), sequence)
    return d

def make_info(f):
    fh = open(f)
    d = {}
    for l in fh.readlines():
        if not l.strip(): continue
        k,v = map(lambda x: x.strip(), l.split('=', 1))
        d[k]=v
    fh.close()
    return d

def ismeta(potential):
    junk, ext = splitext(potential)
    return ext.endswith('info')

def is_meta_for_object(object_file, potential):
    object_path, junk = splitext(object_file)
    fpath, ext = splitext(potential)
    if object_path == fpath and ismeta(potential):
        return 1

def zodb_walker(root, relative_path):
    for r in relative_path:
        root = getattr(root, r)
    return root
            
def main(config, cmf):

    import os
    prefix = os.environ['INSTANCE_HOME']
    directory = config['directory'].strip()
    directory = os.path.join(prefix, directory)

    fsroot = directory
    fspath = directory.split(os.sep)
    fspathlen = len(fspath)
        
    global _extension_handlers
    
    files = gather(directory)
    files.sort()
    total_idx = len(files)
    lookup = make_hash_lookup(range(total_idx))

    indexes = lookup.keys()
    indexes.sort()

    for i in indexes:
        if lookup[i] is not None:
            continue

        f = files[i]
        
        fpath, ext = splitext(f)

        if ismeta(ext):
            continue

        handler = _extension_handlers.get(ext, None)
        
        if handler is None:
            # getaround templates still in static content
            if ext == '.pt' or ext =='.zpt': continue
            raise AttributeError(
                "no handler found for %s extension type %s"%(
                    f, ext
                    )
                )


        info = None
        for ii in range(-1, 3, 2):
            if is_meta_for_object(f, files[i+ii]):

                info = make_info(files[i+ii])
                lookup[i+ii]=1
                break

        container = zodb_walker(cmf, fpath.split(os.sep)[fspathlen:-1])
	print f
        handler(container, f, info)
        lookup[i]=1


def generic_workflow_handler(actions, object):
    import CMFTasks
    CMFTasks.push_workflow({'actions':','.join(actions)}, object)

def generic_content_handler(type, container,  f, info):

    from ZPublisher.mapply import mapply
    from Acquisition import aq_base
    
    types_tool = container.portal_types
    
    id = splitext(os.path.basename(f))[0]

    types_tool.constructContent(type,
                                container,
                                id)
                                

    object = container._getOb(id)

    mapply(object.editMetadata,
           keyword=info,
           context=info
           )
    
    mapply(object.editCWIPMetadata,
           keyword=info,
           context=info,
           )

    if hasattr(aq_base(object), 'edit'):
	#        print 'editing', object.getPhysicalPath(), object.__class__
        mapply(object.edit,
               keyword=info,
               context=info
               )

    return object

def generic_document_handler(type, format, container, f, info):

    object = generic_content_handler(type, container, f, info)
    
    fh = open(f)
    fs = fh.read()
    fh.close()

    object.edit(text_format=format,
                text=fs)

    return object

def delete_object(container,id):

    del_list = [id,]
    container.manage_delObjects(ids=del_list)
    

def hp_handler(container, f, info):

    object = generic_document_handler(type='Content Area Homepage Document',
                             format='html',
                             container=container,
                             f=f,
                             info=info)

    generic_workflow_handler(['submit', 'approve'], object)

def cd_handler(container, f, info):

    object = generic_document_handler(type='CWIP Document',
                             format='html',
                             container=container,
                             f=f,
                             info=info)

    generic_workflow_handler(['submit', 'approve'], object)


def pt_handler(container, f, info):

    id = splitext(os.path.basename(f))[0]

   
    
    fh = open(f)
    fs = fh.read()
    fh.close()

    try:
        container.manage_addProduct['PageTemplates'].manage_addPageTemplate(id)
    except:
        import sys
        if sys.exc_type == 'Bad Request':
            delete_object(container,id)
            container.manage_addProduct['PageTemplates'].manage_addPageTemplate(id)
        else:
            pass
        
    theZPT = container[id]
    theZPT.write(fs)

    
register_handler('.cd', cd_handler)
register_handler('.hp', hp_handler)
register_handler('.pt', pt_handler)

load_data = main

def test():
    import os
    os.environ['INSTANCE_HOME']='/data/zope'
    import Zope
    cmf = Zope.app().internal
    d = {}
    d['directory']='/data/zope/Products/SanhawkSkins/static_content'
    main(d, cmf)

if __name__ == '__main__':
    test()
