from UserDict import UserDict

class TaskConfiguration(UserDict):
    def __getattr__(self, key):
        return self[key]
            
def grant_permissions_to_role(config, ob):
    """
    Grant permissions given in config to role given in config
    with the Zope role interface.
    """
    # cumulative grant of permissions

    role = config.role.strip()
    permissions = [x.strip() for x in config.permissions.split(',')]

    # all permss currently granted to role
    all_permissions = [p['name'] for p in ob.permissionsOfRole(role) if p['selected']]

    # add new perm to granted perm list
    for p in permissions:
        if p not in all_permissions:
            all_permissions.append(p)

    # add role to allowable roles for each perm, remove it from other perms
    ob.manage_role(role, all_permissions)

    # add role to allowable roles for each perm
    #for p in permissions:
    #    ob.manage_permission(p, role, acquire=1)

def restrict_permissions_to_roles(config, ob):
    "Grant permission to given roles only."
    permission = config.permission.strip()
    roles = [x.strip() for x in config.roles.split(',')]
    acquire = config.get("acquire", 0)
    ob.manage_permission(permission, roles, acquire)

def create_role(config, ob):

    role = config.role.strip()
    ob._addRole(role)
    grant_permissions_to_role(config, ob)
    
def remove_permissions_on_role(config, ob):
    """
    Remove permissions given in config to role given in config
    with the Zope role interface.
    """
    
    role = config.role.strip()
    permissions = [x.strip() for x in config.permissions.split(',')]

    all_permissions = [p['name'] for p in ob.permissionsOfRole(role) if p['selected']]

    for p in permissions:
        if p in all_permissions:
            all_permissions.remove(p)

    ob.manage_role(role, all_permissions)

    
def add_xuf_user(config, object):
    "Add a user to the portal's exUserFolder."

    xuf = object.portal_url.getPortalObject().acl_users
    
    # xuf wants to acquire REQUEST. :-(
    from Acquisition import Implicit
    class container(Implicit):
        REQUEST = {}

    req_container = container()
    xuf = xuf.__of__(req_container)

    username = config.username.strip()
    users = xuf.listUsers()
    # XXX if the user exists, we shortcircuit and go
    # away. might need to revisit
    unames = {}
    
    for u in users:
        unames[u['username']]=None
    if unames.has_key(username):
        return

    
    # this should be the raw string
    # under current setup it will be stored as the md5 hash
    passwd   = config.password
    roles    = filter(None, [r.strip() for r in config.roles.split(',')])

    # properties should be prefixed with user_
    from copy import deepcopy
    # because xuf is assuming and make an ass out of itself
    d = {}
    d.update(deepcopy(config.data))
    
    d['REQUEST']={} 
    for n in ('password', 'roles', 'username'):
        try:
            del d[n]
        except:
            pass

    # beware
    xuf._doAddUser(username, passwd, roles, **d)

def add_basic_user(config, ob):
    uf = ob.portal_url.getPortalObject().acl_users
    username = config.username.strip()
    passwd = config.password.strip()
    roles = filter(None, [r.strip() for r in config.roles.split(',')])

    uf._doAddUser(username, passwd, roles, [])

def add_xuf_sanhawk_user(config, ob):
    #import pdb; pdb.set_trace()

    add_xuf_user(config, ob)
    add_cmf_user(config, ob)

def add_basic_sanhawk_user(config, ob):
    add_basic_user(config, ob)
    add_cmf_user(config, ob)

def add_local_roles(config, ob):
    username = config.username.strip()
    local_roles = filter( None, [r.strip() for r in config.local_roles.split(',')])
    ob.manage_addLocalRoles(username, local_roles)

def add_cmf_user(config, ob):
    username = config.username.strip()
    password   = config.password
    properties = {}
    
    memberTool = ob.portal_url.getPortalObject().portal_membership
    regTool = ob.portal_url.getPortalObject().portal_registration
    
    # It would be better to do it the expected way, but we'd have to
    # blow away the acl_users user.  So, do what addMember does.
    #ob.portal_registration.addMember(username, password, roles, '', d)

    # create member by getting it
    member = memberTool.getMemberById(username)
    member.setMemberProperties(properties)
    regTool.afterAdd(member, username, password, properties)

def add_basic_sanhawk_user_with_local_roles(config, ob):
    add_basic_sanhawk_user(config, ob)
    add_local_roles(config, ob)

def add_xuf_sanhawk_user_with_local_roles(config, ob):
    add_xuf_sanhawk_user(config, ob)
    add_local_roles(config, ob)
