"""
a progamatic construction approach to cvs

CvsCLI

  - CvsNoArgOption
  
  - CvsCommand
  
     - CvsArgOption

     - CvsNoArgOption

add options to command, add commands to cli

building a query, pass in the command, and options,

sniff the cmd args... for extra options

arg options gets name maps for readability and to clear
ambiguity.
"""

import sys
import getopt

class CvsCommand:

    def __init__(self, name, short_name):
        self.name = name
        self.short_name = short_name
        self._noarg_options = {}
        self._options = {}
        self._map = {}

    def getName(self):
        return self.name

    def addOption(self, option):

        if self._map.has_key(option.getAlias()):
            raise SyntaxError("duplicate option aliases")

        arg = 0
        if option.arg_p:
            self._options[option.getId()] = option
            arg = 1            
        else:
            self._noarg_options[option.getId()]=option
            
        self._map[option.getAlias()] = option.getId(), arg

    def addOptions(self, *args):
        for a in args:
            self.addOption(a)

    def getArgOptions(self):
        return self._options.values()

    def getNoArgOptions(self):
        return self._noarg_options.values()

    def getOptionForAlias(self, alias):
        option_id, arg_p = self._map.get(alias, (None, None))
        if option_id is None:
            return (None, None)
        if arg_p:
            option = self._options.get(option_id)
        else:
            option = self._noarg_options.get(option_id)
        return option, arg_p

    def getAliases(self):
        res = []
        for k,v in self._map.items():
            res.append( (k, v[1]) )
        return res

    def getOption(self, name):
        opt = self._options.get(name)
        if opt is None:
            return self._noarg_options.get(name)
        return opt
            
    def render(self, arg='', **kw):
        #import pdb; pdb.set_trace()
        arg_res = []
        res = []
        for k,v in kw.items():
            option, argp  = self.getOptionForAlias(k)
            if option is None:
                continue
            if argp:
                arg_res.append(option.render(v))
            else:
                res.append(option.render())

        if res:
            noarg_options = " %s"%' '.join(map(lambda x: "-%s"%x, res))
            #noarg_options = " -%s"%"-%s"%''.join(res)
        else:
            noarg_options = ""

        if arg_res:
            arg_options = " %s"%' '.join(map(lambda x: "-%s"%x, arg_res))
        else:
            arg_options = ''
        return "%s %s%s %s"%(self.name,
                              noarg_options.strip(),
                              arg_options,
                              arg)
                              
                           

class Option: 
    # more of interface than an impl
    def __init__(self, id, alias):
        self.id = id
        self.alias = alias

    def getAlias(self):
        return self.alias

    def getId(self):
        return self.id

    def render(self):
        raise NotImplemented ( " subclass responsiblity " )

class ShortOption(Option): pass
        
class NoArgOption(ShortOption):
    arg_p = 0
    def render(self):
        return self.getId()

class ArgOption(ShortOption):
    arg_p = 1
    def render(self, value):
        if len(value.split()) > 1:
            return '%s "%s"'%(self.getId(), str(value))
        return "%s %s"%(self.getId(), str(value))

class CvsArgOption(ArgOption): pass
class CvsNoArgOption(NoArgOption): pass

class CvsCLI(CvsCommand):

    def __init__(self):
        self.name = 'cvs'
        CvsCommand.__init__(self, self.name, self.name)
        self._commands = {}

    def addCommand(self, command):
        self._commands[command.getName()]=command

    def get_commandline_options(self, aliases):
        res = []
        for name, arg in aliases:
            if arg:
                res.append("%s="%name)
            else:
                res.append(name)
        return getopt.getopt(sys.argv[1:], "", res)[0]
        
    def build_command(self, command, arg='', **kw):

        command = self._commands[command]
        aliases = command.getAliases()

        optlist = self.get_commandline_options(aliases)
        cli_update = {}
        for k,v in optlist:
            cli_update[k[2:]]=v
            
        kw.update(cli_update)
        rendered_command = "%s %s"%(self.render(**kw), command.render(arg,**kw))
        #print 'command', rendered_command
        return  rendered_command

#################################
# now configure the object model

cvs  = CvsCLI()

cvs.addOptions(
    CvsNoArgOption("Q", "cvs_silent"),
    CvsNoArgOption("q", "cvs_quiet"),
    CvsNoArgOption("n", "cvs_no_touch")
    )

# add compress options

for n in range(1, 9):
    cvs.addOption(CvsNoArgOption("z%d"%n, "cvs_compress%d"%n))

#################################
# commands    

checkout = CvsCommand('checkout', 'co')
checkout.addOptions(
    CvsNoArgOption("P", "cvs_purge"),
    #CvsNoArgOption("d", "cvs_newdirs")
    CvsArgOption(  "d", "cvs_rename"),
    CvsArgOption(  "D", "cvs_date"),
    CvsArgOption(  "r", "cvs_label")
    )

cvs.addCommand(checkout)

if __name__ == '__main__':

    print cvs.build_command('checkout', "cwip/appsvr/instance", cvs_silent=1, cvs_rename="fooz", cvs_purge=1)
