

import sys, os, types

import Context
import Loader
import Task
import TaskRunner
import Environment
import Pools
import Log
import Utils

from Exceptions import EngineException

class IEngine(Utils.Interface):

    def getContext():
        pass

    def loadTasks():
        pass

    def execute():
        pass

    def ignition(**kwargs):
        pass

    def configure(**kwargs):
        pass

class Engine:

    __implements__ = IEngine

    def __init__(self):
        
        self._loaded_environment = None
        self.tasks   = None
        
    def getContext(self):

        context_stack = Context.getContext()
        
        if self._loaded_environment is None:
            return context_stack

        return context_stack.getContextChainByName("Engine")
            
    def loadTasks(self):
        """
        load the specified tasks

        there is alot of work to be done in cleaning this up,
        i took the shortcut, of just putting hacks in here to get
        things working, till i have time to clean up. at least
        most of the hacks are localized to this function...
        """
        context = self.getContext()
        log = Log.getLog(self)
        log.info('loading tasks from %s'%str(context.SourceDirectory))

        pools = Loader.MultiPoolLoader().load(context)
        pools = Utils.sort_by_attr(pools, 'priority')

        runner_factory = TaskRunner.RegistryRunner
        
        for p in pools:
            p.setTaskRunner(runner_factory())

        pools  = map(Task.PoolTask, pools)
        
        # XXX again, since we don't have a config step at the moment
        # for pool contexts to be setup, we explicitly set it
        # it to the entire load context, we are creating lots
        # of leaks most likely :-(...
        
        map(lambda p, c=context: p.setContext(c), pools)
        self.tasks =  Pools.SetTaskPool('TaskPoolSet')
        for p in pools:
            self.tasks.addTask(p)
            
        log.info('%d task pools loaded', len(self.tasks))
        
    def configure(self, **kwargs):
        """
        configure the engine
        """

        # first lets get some logging in effect
        from lib import logging
        logging.fileConfig(Log.log_ini)

        log = Log.getLog(self)
        
        # the rest of this is mainly setting up the stack
        log.info('engine starting')
        log.debug('loading environment')
        
        context_stack = self.getContext()
        context_stack.push(Environment.LocationContextFactory())
        context_stack.push(Environment.EnvironmentContextFactory())
        context_stack.push(Environment.CommandLineContextFactory())
        context_stack.push(Context.Context("engine-args", kwargs))
        
        #context_stack.push(Context.Context("engine", {'engine':self}))
        
        context_stack.push(
            Context.Context("defaults",
                            {"TaskFormat":"ini-dir",
                             "TaskFilePattern":"*ini",
                             "DefaultPoolType":"Graph"})
            )

        # now we define some common chains
        Context.defineContextChain("Engine",
               ['command-line', 'engine-args', 'environment', 'location', 'defaults']
        )

        self._loaded_environment = 1

        # sanity check, we'll just exit if we don't have a config
        engine_chain = self.getContext()
        
        try:
            config_directory = engine_chain.SourceDirectory
            if config_directory is None:
                raise
        except:
            log.critical("no configuration specified, exiting")
            Utils.die()


    def execute(self):
        log = Log.getLog(self)
        if not len(self.tasks) > 0:
            log.warn('nothing to do')
            return

        log.info('executing tasks')

        context = self.getContext()
        runner = TaskRunner.TaskRunner(context)
        self.tasks.setTaskRunner(runner)
        self.tasks.runTasks(context)

    def ignition(self, **kwargs):
        self.configure(**kwargs)
        self.loadTasks()
        self.execute()
        
        log = Log.getLog(self)
        log.info('engine shutdown')
        
run  = Engine().ignition



