
import os
import popen2
from Log import getLog

def build_command(format, outputfile=None, **kw):

    command = 'dot'
    
    format_types = {'png':'-Tpng',
                    'ps':'-Tps',
                    'gif':'-Tgif'}

    format_option = format_types.get(format, None)
    
    if format_option is None:
        raise AttributeError ("Invalid Graphviz format %s"%str(format))

    if outputfile is not None:
        output_option = "-o %s"%outputfile
    else:
        output_option = ''

    return '%s %s'%(command, ' '.join( (format_option, output_option) ))


def draw(struct_type, structure, **kw):

    global _structhandler
    
    handler = _structhandler[struct_type]
    command = build_command(**kw)
    dot_serialization = handler(structure)

    return driver(dot_serialization, command)
    

def driver(dot_serialization, command, **kw):

    read_output_p = not kw.get('outputfile', None)

    (read, write, error) = popen2.popen3(command)

    write.write(dot_serialization)
    write.close()

    # overly simplistic
    output_buffer = read.read()
    read.close()

    error_buffer  = error.read()
    error.close()

    if error_buffer:
        # bad..
        #log = getLog(driver)
        #log.warning(error_buffer)
        print 'e', error_buffer
    
    if read_output_p:
        return output_buffer

def draw_graph(graph):

    set = graph.getNodeIds()
    buf = []
    w = buf.append
    
    for n in set:
        for d in graph.getAdjacent(n):
            w("%s -> %s"%(str(n),str(d)))

    body = ';\n    '.join(buf)

    dot_serialization= """
    digraph Engine {
       %s;
    }
    """%body

    return dot_serialization

def draw_tree(tree):
    pass

def draw_set(set):
    pass


_structhandler = {
    'graph':draw_graph,
    'tree':draw_tree,
    'set':draw_set
    }


if __name__ == '__main__':

    class Node:

        def __init__(self, id, adj):
            self.id = id
            self.adj = adj
        def getId(self):
            return self.id
        def getEdges(self):
            return self.adj
            
    import Graph

    g = Graph.Graph()
 
    node_set = [
        Node(1, (2,3,4) ),
        Node(2, (4, 5) ),
        Node(3, (2, 4) ),
        Node(4, (5,) )
        ]

    for n in node_set:
        g.addNode(n)

    draw('graph', g, format='png', outputfile='graph.png')

   

           
