from inspect import getargspec
from Context import Context
from Task    import getTask
from Reader  import getReader
from Pools   import getPool
from Log     import getLog
from Utils   import default_identifier, default_importer, Interface
from lib.mapply  import mapply


from UserDict import UserDict

class ILoader(Interface):

    def load(context):
        """
        only public method on all loaders,
        return val is determined by loader class
        """
        
class ITaskLoader(ILoader):
    """
    returns a set of tasks on load
    """
    
class IPoolLoader(ILoader):
    """
    returns a pool of tasks on load
    """    

class IMultiPoolLoader(ILoader):    
    """
    returns a set of pools on load
    """
    
class BasicTaskLoader:
    """
    acquires reading format and task
    class from context:

    Context Vars: TaskType, TaskFormat
    """

    __implements__ = ITaskLoader
    
    def load(self, context):

        res = []
        
        factory = getTask(context.TaskType)
        reader  = getReader(context.TaskFormat)
        
        reader.read(context)
        
        for tc in reader:
            t = mapply(factory, keyword=tc)
            t.setContext(tc)
            res.append(t)

        return res

class PoolLoader(BasicTaskLoader):

    __implements__ = IPoolLoader

    def load(self, context):

        self.pools = {}

        log     = getLog(self)
        log.debug('Using Context %s', context)
        
        reader  = getReader(context.TaskFormat)()
        log.debug('Using Reader %s', reader)
        
        pool    = getPool(context.PoolType)
        log.debug('Using Pool %s', pool)
        
        pool_id = context.get('pool_id',None) or default_identifier(pool)
        pool = pool(pool_id)
        
        reader.read(context)
        for tc in reader:
            log.debug('context %s', tc)
            object = default_importer(tc.module, tc.task)
            factory = taskSelector(context, tc, object)
            tc['task']=object
            t = mapply(factory, keyword=tc)
            t.setContext(tc)
            log.debug('adding task %s',t)
            pool.addTask(t)
        return pool

class MultiPoolLoader(PoolLoader):

    __implements__ = IMultiPoolLoader

    def load(self, context):

        self.pools = {}

        log = getLog(self)
        
        try:
            tformat = context.TaskFormat
        except:
            tformat = context.DefaultTaskFormat

        log.debug('loading tasks default format selector %s'%tformat)

        reader  = getReader(tformat)()
        log.debug('using reader %s'%str(reader))
        
        reader.read(context)
        
        for tc in reader:            
            log.debug('importing %s task from module %s'%(tc.task, tc.module))
            object = default_importer(tc.module, tc.task)

            factory = taskSelector(context, tc, object)
            log.debug('found %s factory for task'%str(factory))
            tc['task']=object

            log.debug('applying task factory and storing context')
            t = mapply(factory, keyword=tc)
            t.setContext(tc)
            
            # XXX this is broken,
            # we really want to load up a dispatcher
            # based on user config, that will do pool selection
            # with the user defining pools ahead of time
            # as is, pools have to be associated to task types
            # or just use a default

            pool_factory = poolSelector(context, t)
            log.debug('found pool factory %s for task'%str(pool_factory))
            
            pool_id = default_identifier(pool_factory)
            log.debug('found identifier %s for pool'%pool_id)
            
            pool = self.pools.get(pool_id, None)
            if pool is None:
                log.debug('no existing pool found, creating new pool')
                pool = self.pools.setdefault(pool_id, pool_factory(pool_id))
                
            pool.addTask(t)        

        log.debug('loaded %d pools'%len(self.pools))
        return self.pools.values()
    

def poolSelector(load_context, task):

    pool_type = load_context.get('PoolTypeFor%s'%task.getType(), None)
    
    if pool_type is None:
        pool_type = load_context['DefaultPoolType']

    return getPool(pool_type) 

def taskSelector(load_context, task_context, object):

    task_type = load_context.get('DefaultTaskType', 'Engine')
    return getTask(task_type)


        
        

        
        


