"""
directed acyclic graph for dependencies.

adjancey list implementation.
     a
   /    \
b <- c -> d
|
e

$Id: Graph.py,v 1.1.1.1 2002/09/18 08:32:41 ender Exp $
"""


class GraphError(Exception): pass

class DuplicateNode(GraphError): pass

class Graph:

    # tree oriented DAG implementation.
    # mainly useful for dependency graphs

    def __init__(self):
        self._parents = {}
        self._children = {}
        self._nodes = {}

    def addNode(self, node):
        """
        addes a node an arc to the graph
        """

        node_id = node.getId()
        parents = node.getEdges()

        if self._nodes.has_key(node_id):
            raise DuplicateNode("%s node already exists"%str(node_id))

        for p in parents:
            self._parents.setdefault(p, []).append(node_id)

        self._children[node_id] = parents
        self._nodes[node_id]=node

    def getAdjacent(self, node_id):
        """
        returns the directed nodes that can lead to this node.
        """

        return tuple(self._children.get(node_id, ()))

    def getTransitive(self, node_id, node_set=None):
        """
        """

        if node_set is None:
            node_set = [node_id]

        for n in self._children.get(node_id, ()):
            node_set.append(n)
            self.getTransitive(n, node_set)

        return node_set
        
    def getNode(self, node_id):

        return self._nodes[node_id]

    def getNodeIds(self):

        return self._nodes.keys()
