class TreeException(Exception): pass

class Tree:
    
    """
    
    root node has a null for parents

    each node is unique

    multiple independent subtrees allowed
    """

    
    def __init__(self, parents={}, childs={}, data={}):

        self.parents = parents
        self.childs  = childs
        self.data    = data
        
    def addNode(self, node):

        node_id = node.getId()
        parents = node.getEdges()

        self.parents[node_id]=parents

        for p in parents:
            self.childs.setdefault(p, []).append(node_id)

        self.data[node_id]=node

    def getNode(self, node_id):
        return data[node_id]
        
    def getParent(self, node):
        return self.parents[node]
    
    def hasParent(self, node):
        return self.parents.has_key(node)
    
    def getChildren(self, node):
        return self.childs[node]
    
    def hasChild(self,node):
        return self.childs.has_key(node)

    def getLeafNodes(self):
        
        return [n for n in self.parents.keys() if not self.childs.has_key(n)]

    def getAncestors(self, node):
        """
        return all parents of a given node
        """
        res = []
        
        while 1:            
           p = self.parents[node]
           if not p:
               break
           res.append(p)
           node = p
               
        return res    

    def getParentsForSet(self, set):

        return [self.get_parent(s) for s in set]

    def getSubTree(self, node):
        """  include node :( """

        res  = []

        res.append(node)
        
        if not self.has_child(node):
            return res

        for n in self.getChildren(node):
            res.extend(self.get_sub_tree(n))

        return res

    def getDescendants(self, node, level=-1):

        res = []
        res.append(node)
        
        if not self.hasChild(node) or level == 0:
            return res
        
        for n in self.getChildren(node):
            res.extend(self.getDescendants(n, level-1))
                
        return res
        
    def getRootNodes(self):

        res = {}

        for n in self.getLeafNodes():
            res[self.getAncestors(n).pop()]=None

        return res.keys()
             
    def removeNode(self, node, ignore_set=()):
        """
        removes a node from a tree, performing the
        proper manipulation to add the subtree descendant
        from the node to an ancestor node. nodes in the ignore
        set are not valid for ancestors.
        """        

        if self.hasChild(node):
            children = self.childs[node]
            del self.childs[node]            
        else:
            children = []

        target = node
    
        while 1:
            ancestor = self.parents[target]
            if not ancestor in ignore_set:
                break
            target = ancestor

        if not ancestor:
            raise TreeException("Cannot Remove Root Node")
        
        for c in children:
            self.parents[c]=ancestor

        self.childs[ancestor].extend(children)

        parent = self.parents[node]
        del self.parents[node]
        
        parent_children = self.childs[parent]
        parent_children.remove(node)

        if not parent_children:
            del self.childs[parent]

    def removeNodeSet(self, set):
        """
        for removing a set of nodes from the tree
        """

        for r in set:
            self.removeNode(r, set)

    def __str__(self):

        return str( str(self.childs), str(self.parents))

def test():

    parents = {1:None,
               2:1,
               3:2,
               4:1,
               5:3,
               6:3,
               7:2,
               8:7,
               9:8}

    children = {1:[2,3,4],
                2:[3,7],
                3:[5,6],
                7:[8],
                8:[9]                
                }

    t = Tree(parents, children)

    print 'leaf'
    print t.get_leaf_nodes()

    print 'root'
    print t.get_root_nodes() 

    print '9 ancestors'
    print t.get_ancestors(9)

    print '1 subtree'
    print t.get_sub_tree(1)

    print '2 descendants'
    print t.get_descendants(2, 1)

    print 'remove node 4'
    t.remove_node(4)

    print '1 children'
    print t.get_children(1)

    print 'remove set (2,3)'
    t.remove_node_set( (2,3) )

    print '1 children'
    print t.get_children(1)

                            
if __name__ == '__main__':
    import sys
    sys.setrecursionlimit(40)
    test()


