from Utils import Interface
from Exceptions import TaskRunnerException
from lib.mapply import mapply
from Log import getLog

#################################
# the name of the value passed in by the
# runner. dependent on what has run before.
# should be customized on the subclass
RUNVALUE = 'run_value'

class ITaskRunner(Interface):

    def executeTasks(self, execution_context, tasks):
        """
        execute the given tasks
        """

#################################
# while there are alot of subclasses
# its probably easiest to just write a
# from scratch implementation if you need
# more than whats provided, instead of
# shoehorning.

class RunnerBase:
    # just an example
    def __init__(self):
        
        self.tasks = []

    def executeTasks(self, execution_context, tasks):

        for t in tasks:
            v = self._get_val(execution_context, t)
            execution_context['run_value']=v
            r = self._execute(execution_context, t)
            self._save_val(t, r)

    def _save_val(self, t):
        raise NotImplemented (" Subclass Responsibility ")
    def _get_val(self, t, v):
        raise NotImplemented (" Subclass Responsibility ")        
    def _execute(self, context, t, val=None):
        raise NotImplemented (" Subclass Responsibility ")        

#################################
# class options for specifying how we
# handle task return values

class RegistryRunner:

    RUNID    = 'objectbytaskid'
    RUNVALUE = RUNVALUE
    
    def __init__(self):
        self._return_registry = {}

    def executeTasks(self, execution_context, tasks):

        log = getLog(self)
        
        for t in tasks:
            
            log.debug('Executing %s', t)
            
            tid = t.getContext().get(self.RUNID, None)
            if tid is not None:
                oval = self._return_registry.get(tid, None)
                execution_context[self.RUNVALUE]=o

            execution_context.push(t.getContext())

            rval = t(execution_context)
            self._return_registry[t.getId()]=rval
            execution_context.pop()
            execution_context.clear() # clear out return value in chain
            
class PipeRunner:

    RUNVALUE=RUNVALUE
    
    def __init__(self):
        self._last_val = None

    def executeTasks(self, execution_context, tasks):
        for t in tasks:
            execution_context[self.RUNVALUE]=self._last_val
            execution_context.push(t.getContext())
            self._last_val = t(execution_context)
            execution_context.pop()
            execution_context.clear() # clear out return value in chain        


# backwards compatiblity

class ZopeTaskRunner(RegistryRunner):

    RUNVALUE = 'objectbytaskid'

class TaskRunner:

    def __init__(self, config):
        self.config = config
        self._return_registry = {}

    def executeTasks(self, execution_context, tasks):
        log = getLog(self)

        for t in tasks:
            log.info('Executing %s', t)

            tconf = t.getContext()
            
            r = t(tconf)
            self._return_registry[t.getId()]=r





        
