from unittest import TestCase, TestSuite, main, makeSuite

from TaskEngine.Context import Context, ContextChain, ContextStack

class ContextTests(TestCase):

    def testGetattr(self):

        c = Context('baz', {'d':1})
        assert c.d == 1, "bad lookup"

class ContextChainTests(TestCase):

    def setUp(self):

        ca = self.ca = Context('aaa', {'myname':'max'})
        cb = self.cb = Context('bbb', {'food':'pizza'})
        cc = self.cc = Context('ccc', {'myname':'fooz'})

        id = lambda x: x.id
        
        self.ch = ContextChain('anon-chain',
                               map(id, (ca, cb, cc)),
                               (ca,cb,cc)
                               )

    def tearDown(self):
    
        del self.ca 
        del self.cb
        del self.cc
        del self.ch
        
    def testLookupOrder(self):
        assert self.ch.myname == 'max', "wrong lookup order"

    def testLookupOrderWithChainValue(self):
        self.ch['food']='guiness'
        assert self.ch.food == 'guiness', 'wrong lookup order for chain value'

    def testPushContext(self):
        self.ch.push(Context('muahha', {'food':'garlic'}))
        assert self.ch.food == 'garlic', 'wrong lookup order for pushed context'

    def testPopContext(self):
        c = self.ch.pop()
        assert self.ch.myname == 'fooz', 'wrong lookup order after pop'

    def testPopContext2(self):
        c = self.ch.pop(1)
        assert c == self.cb, "wrong popped context"
        f = getattr(self.ch, 'food', None)
        assert f is None, "popped context still in chain"

    def testPopContext3(self):
        c = self.ch.popContext("bbb")
        assert c == self.cb, "wrong popped context"
        f = getattr(self.ch, 'food', None)
        assert f is None, "popped context still in chain"        

    def testGetContext(self):
        c = self.ch.getContext('ccc')
        assert c == self.cc, "not equal"


class ContextStackTests(TestCase):

    def setUp(self):

        ca = self.ca = Context('aaa', {'myname':'max'})
        cb = self.cb = Context('bbb', {'food':'pizza'})
        cc = self.cc = Context('ccc', {'myname':'fooz'})

        self.cs = ContextStack('stack')
        map(cs.push, (ca, cb, cc))

    def tearDown(self):
    
        del self.ca 
        del self.cb
        del self.cc
        del self.cs
        
    def testGetContextChainExplicit(self):

        chain = self.cs.getContextChain( ('cc','ca') )
        assert chain.myname == 'fooz', "bad chain lookup"


    def testGetContextChainImplied(self):

        chain = self.cs.getContextChain( ('cc','ca'), explicit_order=0 )
        assert chain.myname == 'max', "bad chain lookup"        
        
        
def test_suite():
    return TestSuite((
        makeSuite(ContextTests),
        makeSuite(ContextChainTests)
        ))

if __name__=='__main__':
    main(defaultTest='test_suite')

