from __future__ import nested_scopes

from unittest import TestCase, TestSuite, main, makeSuite

from TaskEngine.Structure.Graph import Graph

class GraphTests(TestCase):

    class Node:

        def __init__(self, id, parents):
            self.id = id
            self.parents = parents
            
        def getId(self):
            return self.id

        def getEdges(self):
            return self.parents
            
    def setUp(self):

        self.node_set = [ self.Node(1, ()),
                          self.Node(2, (1,)),
                          self.Node(3, (2,1)),
                          self.Node(4, (2,3)),
                          self.Node(5, (1,)),
                          self.Node(6, ()),
                          self.Node(7, (3,4,5))
                     ]

        g = Graph()
        map(g.addNode, self.node_set)
        self.graph = g
        
        
    def tearDown(self):
        self.graph = None
        self.node_set = None

    def testNodeAddition(self):

        g = Graph()
        map(g.addNode, self.node_set)

        node_ids = g.getNodeIds()
        node_ids.sort()

        test_ids = range(1, len(self.node_set)+1)

        self.assertEqual(node_ids, test_ids, "Missing Node During Addition")

    def testNodeAdjancey(self):

        self.assertEqual(self.graph.getAdjacent(7), (3,4,5),
                         "Failed Node Adjancey Test")

    def testGetTransitive(self):

        self.assertEqual(self.graph.getTransitive(6), [6],
                         """ Bad Get Transitive (1) """)

        self.assertEqual(self.graph.getTransitive(5), [5,1],
                         """ Bad Get Transitive (2) """)

        self.assertEqual(self.graph.getTransitive(7), [7, 3, 2, 1, 1, 4, 2, 1, 3, 2, 1, 1, 5, 1],
                         """ Bad Get Transitive (3) """)
    

def test_suite():
    return TestSuite((
        makeSuite(GraphTests),
        ))

if __name__=='__main__':
    main(defaultTest='test_suite')

