from unittest import TestCase, TestSuite, main, makeSuite

class PoolTests(unittest.TestCase):

    def setUp(self):
        from Context import Context
        from Task import Task

        class testtask(Task):
            __implements__ = ITask
            ran = 0
            def __call__(self, context):
                self.ran = 1
                context.setdefault('order', []).append(self.id)

        class testrunner:
            def executeTasks(self, ctx, tasks):
                for t in tasks:
                    t(ctx)
                
        self.tasks = [
            testtask('t1'),
            testtask('t2', ('t1',) ),
            testtask('t3', ('t2', 't4') ),
            testtask('t4', ('t1', 't2') )
            ]

        self.context = Context('context')
        self.pool = None
        self.runner = testrunner
        
    def tearDown(self):
        del self.tasks
        del self.context
        del self.pool
        del self.runner

    def poolTest(self):
        self.pool.setTaskRunner(self.runner())
        for t in self.tasks:
            self.pool.addTask(t)
        self.pool.runTasks(self.context)

    def testGraphPool(self):
        self.pool = GraphTaskPool('pool')
        self.poolTest()
        self.assertEqual( tuple(self.context['order']),
                          ('t1', 't2', 't4', 't3')
                          )
        for t in self.tasks:
            self.assertEqual(t.ran, 1)
        
    def testSetPool(self):
        self.pool = SetTaskPool('pool')
        self.poolTest()
        self.assertEqual(tuple(self.context['order']),
                         ('t1', 't2', 't3', 't4')
                         )
        for t in self.tasks:
            self.assertEqual(t.ran, 1)
            
    def testTreePool(self):
        self.pool = TreeTaskPool('pool')
        #self.poolTest()
        
def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(PoolTests),
        ))

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')

