from unittest import TestCase, TestSuite, main, makeSuite

from CSInstaller.Task import Task, ZopeTask
from CSInstaller.TaskRegistry import TaskRegistry, addTask

class TaskRegistryTests(TestCase):

    
    
    def setUp(self):

        self.task_set = [Task(1, ()),
                         Task(2, (1,)),
                         Task(3, (2,)),
                         Task(4, (1,)),
                         Task(5, (3,1)),
                         Task(6, ()),
                         Task(7, (5,6)),
                         ZopeTask(20, ()),
                         ZopeTask(21, (20,))
                         ]

        map(addTask, self.task_set)
        
    def tearDown(self):
        
        self.graph = None
        self.node_set = None
        TaskRegistry.clear()

    def testAddTask(self):

        TaskRegistry.clear()
        TaskRegistry.addTask(self.task_set[0])
        self.assertEqual(len(TaskRegistry.getTaskList()), 1,
                         " task addition failed ")

    def testAddTaskWithModeInconsistencies(self):

        try:
            addTask(ZopeTask(22, 1))
        except:
            return

        self.fail( "Allowed Mode Inconsistencies in Task Addition" )
        
    def testGetTaskList(self):

        self.assertEqual(len(TaskRegistry.getTaskList(mode='environ')),
                         7,
                         " task get list failed ")


        self.assertEqual(len(TaskRegistry.getTaskList(mode='zope')),
                         2,
                         " task get list failed (2)")

    def testGetTaskListOrder(self):


        self.assertEqual(TaskRegistry.getTaskList(mode='zope'),
                         [20,21],
                         " task order wrong ")

        # technically this could go be [6,1,2,3,5,7] as well
        self.assertEqual(TaskRegistry.getTaskList(mode='environ'),
                         [1,2,3,4,5,6,7]
                         )

def test_suite():
    return TestSuite((
        makeSuite(TaskRegistryTests),
        ))

if __name__=='__main__':
    main(defaultTest='test_suite')

